package ru.dbotthepony.kstarbound

import org.apache.logging.log4j.LogManager
import ru.dbotthepony.kstarbound.defs.ActorMovementParameters
import ru.dbotthepony.kstarbound.defs.ClientConfigParameters
import ru.dbotthepony.kstarbound.defs.MovementParameters
import ru.dbotthepony.kstarbound.util.AssetPathStack
import java.util.concurrent.ExecutorService
import java.util.concurrent.ForkJoinTask
import java.util.concurrent.Future
import kotlin.reflect.KMutableProperty0

object GlobalDefaults {
	private val LOGGER = LogManager.getLogger()

	var actorMovementParameters = ActorMovementParameters()
		private set

	var movementParameters = MovementParameters()
		private set

	var clientParameters = ClientConfigParameters()
		private set

	private object EmptyTask : ForkJoinTask<Unit>() {
		private fun readResolve(): Any = EmptyTask
		override fun getRawResult() {
		}

		override fun setRawResult(value: Unit?) {
		}

		override fun exec(): Boolean {
			return true
		}
	}

	private inline fun <reified T> load(path: String, accept: KMutableProperty0<T>, executor: ExecutorService): Future<*> {
		val file = Starbound.locate(path)

		if (!file.exists) {
			LOGGER.fatal("$path does not exist, expect bad things to happen!")
			return EmptyTask
		} else if (!file.isFile) {
			LOGGER.fatal("$path is not a file, expect bad things to happen!")
			return EmptyTask
		} else {
			return executor.submit {
				AssetPathStack("/") {
					accept.set(Starbound.gson.fromJson(file.jsonReader(), T::class.java))
				}
			}
		}
	}

	fun load(executor: ExecutorService): List<Future<*>> {
		val tasks = ArrayList<Future<*>>()

		tasks.add(load("/default_actor_movement.config", ::actorMovementParameters, executor))
		tasks.add(load("/default_movement.config", ::movementParameters, executor))
		tasks.add(load("/client.config", ::clientParameters, executor))

		return tasks
	}
}
