package ru.dbotthepony.kstarbound.client.network.packets

import ru.dbotthepony.kstarbound.network.IServerPacket
import ru.dbotthepony.kstarbound.server.network.ServerConnection
import ru.dbotthepony.kstarbound.util.readVec2d
import ru.dbotthepony.kstarbound.util.writeVec2d
import ru.dbotthepony.kvector.vector.Vector2d
import java.io.DataInputStream
import java.io.DataOutputStream

data class TrackedPositionPacket(val pos: Vector2d) : IServerPacket {
	constructor(stream: DataInputStream) : this(stream.readVec2d())

	override fun write(stream: DataOutputStream) {
		stream.writeVec2d(pos)
	}

	override fun play(connection: ServerConnection) {
		connection.player.trackedPosition = pos
	}
}

data class TrackedSizePacket(val width: Int, val height: Int) : IServerPacket {
	constructor(stream: DataInputStream) : this(stream.readUnsignedByte(), stream.readUnsignedByte())

	init {
		require(width in 0 .. 12) { "Too big chunk width to track: $width" }
		require(height in 0 .. 12) { "Too big chunk height to track: $height" }
	}

	override fun write(stream: DataOutputStream) {
		stream.writeByte(width)
		stream.writeByte(height)
	}

	override fun play(connection: ServerConnection) {
		connection.player.trackedChunksWidth = width
		connection.player.trackedChunksHeight = height
	}
}
