package ru.dbotthepony.kstarbound.server.world

import ru.dbotthepony.kstarbound.io.BTreeDB
import ru.dbotthepony.kstarbound.util.KOptional
import ru.dbotthepony.kstarbound.world.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.ChunkPos
import ru.dbotthepony.kstarbound.world.api.AbstractCell
import ru.dbotthepony.kstarbound.world.api.MutableCell
import ru.dbotthepony.kstarbound.world.entities.WorldObject
import ru.dbotthepony.kvector.arrays.Object2DArray
import java.io.BufferedInputStream
import java.io.ByteArrayInputStream
import java.io.DataInputStream
import java.util.concurrent.CompletableFuture
import java.util.zip.Inflater
import java.util.zip.InflaterInputStream

class LegacyChunkSource(val db: BTreeDB) : IChunkSource {
	override fun getTiles(pos: ChunkPos): CompletableFuture<KOptional<Object2DArray<out AbstractCell>>> {
		val chunkX = pos.x
		val chunkY = pos.y
		val key = byteArrayOf(1, (chunkX shr 8).toByte(), chunkX.toByte(), (chunkY shr 8).toByte(), chunkY.toByte())
		val data = db.read(key) ?: return CompletableFuture.completedFuture(KOptional.empty())

		val reader = DataInputStream(BufferedInputStream(InflaterInputStream(ByteArrayInputStream(data), Inflater())))
		reader.skipBytes(3)

		val result = Object2DArray.nulls<MutableCell>(CHUNK_SIZE, CHUNK_SIZE)

		for (y in 0 until CHUNK_SIZE) {
			for (x in 0 until CHUNK_SIZE) {
				result[x, y] = MutableCell().read(reader)
			}
		}

		return CompletableFuture.completedFuture(KOptional(result as Object2DArray<out AbstractCell>))
	}

	override fun getObjects(pos: ChunkPos): CompletableFuture<KOptional<Collection<WorldObject>>> {
		return CompletableFuture.completedFuture(KOptional.of(listOf()))
	}
}
